const FilterPoint = function(params = {}) {
	const {
		map
	} = params;
	this.map = map;
	this.SPORT_MAX_SPEED = 20; // 一秒的距离 (m)
	this.isFirst = 0;
	this.w1Count = 0;
	this.weight2 = null;
	this.weight1 = null;
	this.w1TempList = [];
	this.w2TempList = [];
	this.accuracyCount = 0;
	// 计算纠偏用的参数
	this.LOCATION_COUNT = 2; // 抛弃的次数
	this.MIN_DISTANCE = 5; // 最小更新距离
	this.MAX_DISTANCE = 150; // 最大更新距离
	this.MIN_OFFSET_TIME = 0; // 最小偏移时间
	this.MAX_SUCCESS_COUNT = 1; // 成功几次算入成功的点位
	this.MAX_ERROR_COUNT = 3; // 失败几次算入成功的点位
	this.MAX_SUCCESS_ERROR_COUNT = 2; // 计入偏移过程之前可以放入的数量
	this.accuracymax = 10;
	this.ACCURACYMAX = this.accuracymax; // 获取的最大精度
	this.ACCURACYMAXCOUNT = 5; // 精度误差
	this.ACCURACYMAX_ERROR = 30; // 精度误差次数
}
/**
 * @description 处理位置信息
 * @param {Object} aMapLocation
 * @param {Object} currentStamp
 */
FilterPoint.prototype.getLocation = function(aMapLocation, currentStamp) {
	const {
		lo,
		coords: {
			accuracy,
			longitude,
			latitude,
		},
		addresses
	} = aMapLocation;
	const [lng, lat] = lo;;
	return {
		ts: new Date().format('yyyy-MM-dd hh:mm:ss'),
		lng,
		lat,
		latitude: lat,
		longitude: lng,
		latitude_accurate: latitude,
		longitude_accurate: longitude,
		coord_type_input: 'gcj02',
		timeStamp: currentStamp,
		loc_time: currentStamp,
		radius: accuracy,
		address: addresses,
	};
}
/**
 * 判断偏移点算法
 * @param aMapLocation 高德定位返回
 * @return 自定义稳定点的集合
 */
FilterPoint.prototype.insert = function(aMapLocation) {
	const currentStamp = new Date().getTime() / 1000; // 时间戳
	const currentPosition = this.getLocation(aMapLocation, currentStamp);
	const {
		radius
	} = currentPosition;

	// 判断精度是否在范围内
	if (radius > this.ACCURACYMAX && radius - this.ACCURACYMAX > this.ACCURACYMAXCOUNT) {
		this.accuracyCount += 1;
		if (this.accuracyCount === this.ACCURACYMAX_ERROR) {
			// alert(`精度不够超过${this.ACCURACYMAX_ERROR}次`);
			this.accuracyCount = 0;
			this.ACCURACYMAX = radius + this.ACCURACYMAXCOUNT;
		}
		// console.log('精度不够' + this.accuracyCount);
		return null;
	} else {
		this.accuracyCount = 0;
	}

	const {
		lo
	} = aMapLocation;
	const [lng, lat] = lo;

	// 获取的第一个定位点不进行过滤
	// 第一个定位点容易漂移不记录，不更新
	if (this.isFirst < this.LOCATION_COUNT) {
		this.isFirst++;

		if (this.isFirst >= this.LOCATION_COUNT) {
			this.weight1 = {
				...currentPosition
			};
			// 将得到的第一个点存储入w1的缓存集合
			return [this.weight1];
		}

		return null;
	}



	// 进入判断条件
	if (!this.weight2) {
		// 偏移时间
		const offsetTime = currentStamp - this.weight1.timeStamp;

		if (offsetTime <= this.MIN_OFFSET_TIME) {
			return null;
		}

		// 最大偏移距离D
		const MaxDistance = offsetTime * this.SPORT_MAX_SPEED;
		// 当前点与上一个点位偏移距离
		// 计算w1与当前定位点p1的时间差并得到最大偏移距离D
		const {
			lng: wlng1,
			lat: wlat1,
		} = this.weight1;
		const w1Position = [wlng1, wlat1];
		const distance = this.map.GeometryUtil.distanceOfLine([
			w1Position,
			lo
		]);

		// console.log(MaxDistance + '最大距离')
		// console.log(distance + '上个点的距离')
		console.log('不进入偏移点')

		// 最大距离
		if (MaxDistance >= this.MAX_DISTANCE) {
			// this.weight1.timeStamp = currentStamp;
			// return null;
		}

		// 距离为最小 抛出
		if (distance <= this.MIN_DISTANCE) {
			// this.weight1.timeStamp = currentStamp;
			return null;
		}




		// 大于最大距离 记入偏移点
		if (distance > MaxDistance) {
			// 将设置w2位新的点，并存储入w2临时缓存
			this.weight2 = {
				...currentPosition
			};
			this.w2TempList.push(this.weight2);
			return null;
		}



		// 小于最大距离 正常计入点位
		this.w1TempList.push({
			...currentPosition
		});
		// 更新w1权值点
		// this.weight1.lat = this.weight1.lat * 0.2 + lat * 0.8;
		// this.weight1.lng = this.weight1.lng * 0.2 + lng * 0.8;
		this.weight1 = {
			...currentPosition
		};
		// this.weight1.timeStamp = currentStamp;

		if (this.w1TempList.length >= this.MAX_SUCCESS_COUNT) {
			const list = [...this.w1TempList];
			this.w1TempList = [];
			return list;
		} else {
			return null;
		}

	} else {
		// 计算w2与当前定位点p1的时间差并得到最大偏移距离D
		const offsetTime = currentStamp - this.weight2.timeStamp;

		if (offsetTime <= this.MIN_OFFSET_TIME) {
			return null;
		}


		const MaxDistance = offsetTime * this.SPORT_MAX_SPEED;
		const {
			lng: wlng2,
			lat: wlat2,
		} = this.weight2;
		const w2Position = [wlng2, wlat2];
		const distance = this.map.GeometryUtil.distanceOfLine([
			w2Position,
			lo
		]);

		// console.log(MaxDistance + '最大距离')
		// console.log(distance + '上个点的距离')
		console.log('进入偏移点')

		// 最大距离
		if (MaxDistance >= this.MAX_DISTANCE) {
			// this.weight2.timeStamp = currentStamp;
			// return null;
		}

		// 静止状态
		if (distance <= this.MIN_DISTANCE) {
			// this.weight2.timeStamp = currentStamp;
			return null;
		}



		// 大于最大距离
		if (distance > MaxDistance) {
			this.w2TempList = [];
			// 将设置w2位新的点，并存储入w2临时缓存
			this.weight2 = {
				...currentPosition
			};
			this.w2TempList.push(this.weight2);
			return null;
		} else {
			// 小于最大距离
			// 将p1加入到做坐标集合w2TempList
			this.w2TempList.push({
				...currentPosition
			});
			// this.weight2.lat = this.weight2.lat * 0.2 + lat * 0.8;
			// this.weight2.lng = this.weight2.lng * 0.2 + lng * 0.8;
			// this.weight2.timeStamp = currentStamp;
			this.weight2 = {
				...currentPosition
			};

			if (this.w2TempList.length >= this.MAX_ERROR_COUNT) {
				this.weight1.lat = this.weight2.lat;
				this.weight1.lng = this.weight2.lng;
				this.weight1.timeStamp = this.weight2.currentStamp;
				this.weight2 = null;
				if (this.w1TempList.length >= this.MAX_SUCCESS_ERROR_COUNT) {
					const list = this.w1TempList.concat(this.w2TempList);
					this.w1TempList = [];
					this.w2TempList = [];

					return list;
				}

				const list = [...this.w2TempList];
				this.w1TempList = [];
				this.w2TempList = [];
				return list;
			}

			return null;
		}
	}

}


/**
 * @description 清除变量
 */
FilterPoint.prototype.clear = function() {
	this.isFirst = 0;
	this.ACCURACYMAX = this.accuracymax;
	this.accuracyCount = 0;
	this.w1Count = 0;
	this.weight2 = null;
	this.weight1 = null;
	this.w1TempList = [];
	this.w2TempList = [];
}
