const defaultConfig = {
	center: [115.2426242, 28.8614724],
	viewMode: '2D',
	zoom: 10,
	resizeEnable: true,
	layers: []
	// rotateEnable: true,
}

let _mapHost;

if (location.pathname.indexOf('www') > -1) {
	_mapHost = location.pathname.split('www')[0] + 'www';
} else {
	_mapHost = location.origin + '/piesat_jax';
}

//定位按钮
const OPTIONS = {
	//'enableHighAccuracy': true,//是否使用高精度定位，默认:true
	'showButton': true, //是否显示定位按钮
	'buttonPosition': 'LB', //定位按钮的位置
	/* LT LB RT RB */
	'buttonOffset': new AMap.Pixel(10, 20), //定位按钮距离对应角落的距离
	'showMarker': true, //是否显示定位点
	'markerOptions': { //自定义定位点样式，同Marker的Options
		'offset': new AMap.Pixel(-18, -36),
		'content': '<img src="https://a.amap.com/jsapi_demos/static/resource/img/user.png" style="width:36px;height:36px"/>'
	},
	'showCircle': true, //是否显示定位精度圈
	'circleOptions': { //定位精度圈的样式
		'strokeColor': '#0093FF',
		'noSelect': true,
		'strokeOpacity': 0.5,
		'strokeWeight': 1,
		'fillColor': '#02B0FF',
		'fillOpacity': 0.25
	},
	'panToLocation': true,
}

function arrayBuffer2Base64(buffer) {
	var binary = '';
	var bytes = Uint8Array.from(buffer);
	var len = bytes.length;
	for (let i = 0; i < len; i++) {
		binary += String.fromCharCode(bytes[i]);
	}
	// return binary.toString("base64");
	return btoa(binary);
};

/**
 * @param {map}  高德实例
 * @param {container}  类名
 * @param {config}  初始化配置项
 */
const InitMap = function({
	map,
	container,
	config,
	options
}) {
	this.mapInstance = map; // 高德地图实例
	this.map = null; // 实例化的map
	this.container = container;
	this.geolocation = null;
	this.config = Object.assign(defaultConfig, config);
	// 定位按钮配置项
	this.options = Object.assign(OPTIONS, options);
	this.infoWindow = new this.mapInstance.InfoWindow({
		offset: new this.mapInstance.Pixel(14, -8)
	});
	// 图层
	this.layerSet = {};
	// 初始化
	this.container && this._init();
	// 事件列表
	this.eventHandle = new Map();
	// 轨迹
	this.endMarker = null;
	this.startMarker = null;
	this.polyLine = null;
	// this.iconUrl = '../../images/map/icon_dir-marker.png'
	// https://a.amap.com/jsapi_demos/static/demo-center/icons/dir-marker.png
	this.iconUrl = `${_mapHost}/images/map/icon_dir-marker.png`;
	// 创建一个 Icon
	this.startIcon = new this.mapInstance.Icon({
		// 图标尺寸
		size: new AMap.Size(25, 34),
		// 图标的取图地址
		image: this.iconUrl,
		// 图标所用图片大小
		imageSize: new AMap.Size(135, 40),
		// 图标取图偏移量
		imageOffset: new AMap.Pixel(-9, -3)
	});
	this.endIcon = new this.mapInstance.Icon({
		size: new AMap.Size(25, 34),
		image: this.iconUrl,
		imageSize: new AMap.Size(135, 40),
		imageOffset: new AMap.Pixel(-95, -3)
	});
}
// 初始化
InitMap.prototype._init = function() {
	let {
		mapInstance,
		container,
		config
	} = this;

	this.config.layers = this.initLayer();

	this.map = new mapInstance.Map(container, config);

}


/**
 * @description  设置地图可控范围
 */
InitMap.prototype.setLimitBounds = function() {
	const southWest = new this.mapInstance.LngLat(114.74609847143601, 28.679184393985356);
	const northEast = new this.mapInstance.LngLat(115.69762455279913, 29.13383001510663);
	const bounds = new this.mapInstance.Bounds(southWest, northEast);
	this.map.setLimitBounds(bounds);
}

/**
 * @description 增加定位图标
 */
InitMap.prototype.addLocationIcon = function() {
	const {
		map,
		mapInstance,
		options
	} = this;
	//定位
	mapInstance.plugin(["AMap.Geolocation"], function() {
		this.geolocation = new mapInstance.Geolocation(options);
		map.addControl(this.geolocation);

		//自动定位到目标点
		/* geolocation.getCurrentPosition(
			function(status, result) {
				if (status == 'complete') {
					onComplete(result)
				} else {
					onError(result)
				}
			}); */

	});
}

/**
 * @description  获取当前位置
 * @param {Function} callback 成功之后的回调
 * @param {Function} error 失败之后的回调
 */
InitMap.prototype.getCurrentLocation = function(callback, error) {
	const {
		map,
		mapInstance,
		options
	} = this;
	//定位
	mapInstance.plugin(["AMap.Geolocation"], () => {
		this.geolocation = new mapInstance.Geolocation(options);
		map && map.addControl(this.geolocation);
		this.geolocation.getCurrentPosition(
			function(status, result) {
				if (status == 'complete') {
					callback && callback(result)
				} else {
					error && error(result)
				}
			});
	});
}

/**
 * @description 获取天气预报
 * @param {String} liveName 获取天气的城市名称
 * @param {Function} callback 成功之后的回调
 */
InitMap.prototype.getWeather = function(liveName, callback) {
	const {
		mapInstance
	} = this;
	mapInstance.plugin('AMap.Weather', function() {
		var weather = new mapInstance.Weather();
		//查询实时天气信息, 查询的城市到行政级别的城市，如朝阳区、杭州市
		weather.getLive(liveName, function(err, data) {
			if (!err) {
				callback && callback(data)
			}
		});

	});
}

/**
 * @description 通过名字调用高德加载边界
 * @param {String} name 区域名字
 */

InitMap.prototype.setCityBound = function(name) {
	const {
		mapInstance
	} = this;
	mapInstance.plugin([
		'AMap.DistrictSearch',
	], () => {
		var districtSearch = new mapInstance.DistrictSearch({
			// 关键字对应的行政区级别，共有5种级别
			level: 'province',
			//  是否显示下级行政区级数，1表示返回下一级行政区
			subdistrict: 0,
			// 返回行政区边界坐标点
			extensions: 'all',
		})

		// 搜索所有省/直辖市信息
		districtSearch.search(name || '靖安', (status, result) => {
			// 查询成功时，result即为对应的行政区信息
			// console.log('dd' + status + JSON.stringify(result))
			this.handlePolygon(result)
		})
	});
}


/**
 * @description 添加polygon
 */
InitMap.prototype.handlePolygon = function(result) {
	const {
		mapInstance,
		map
	} = this;
	let bounds = result.districtList[0].boundaries
	if (bounds) {
		for (let i = 0, l = bounds.length; i < l; i++) {
			//生成行政区划polygon
			let polygon = new mapInstance.Polygon({
				map: map, // 指定地图对象
				strokeWeight: 2, // 轮廓线宽度
				strokeOpacity: 0.8,
				path: bounds[i], //轮廓线的节点坐标数组
				fillOpacity: 0.3, //透明度
				strokeColor: '#0055ff',
				fillColor: '#55b4ff',
			})
			polygon.on('click', (e) => {
				this.clearMarkerDetail();
				console.log("点击图层");
				var address;
				var lnglatXY = [e.lnglat.getLng(), e.lnglat.getLat()];
				/* geocoder.getAddress(lnglatXY, function(status, result) {
					if (status === 'complete' && result.info === 'OK') {
						address = result.regeocode.formattedAddress; //返回地址描述
						mui.fire(targetPage, 'eventLocation', {
							address: address,
							lng: e.lnglat.getLng(),
							lat: e.lnglat.getLat()
						});
						ws.close();
					}
				}); */
			})
		}
		// 地图自适应
		// this.map.setFitView()
	}
}

/**
 * @description 添加maker点
 * @param {Object} list 点位数据
 * @param {Object} createInfo 是否添加详情点击事件
 */
InitMap.prototype.addMarker = function(list, createInfo) {
	this.mapInstance.plugin([
		'AMap.Marker',
	], () => {
		list.forEach((item, i) => {
			const {
				position,
				attr,
			} = item;
			var marker = new this.mapInstance.Marker({
				position,
				map: this.map,
				'offset': new this.mapInstance.Pixel(-18, -30),
				content: '<img src="../../../../images/home/icon_site.png" style="transform: scale(0.5);"/>'
				// icon: '../../../../images/home/icon_site.png',
				// scale: 0.5,
			});
			if (createInfo) {
				const content = document.createElement('div');
				Object.entries(attr).forEach((i) => {
					const [type, value] = i;
					const l = document.createElement('div');
					l.innerHTML = type + '：' + value
					content.appendChild(l)
				})
				marker.content = content;
				marker.on('click', (e) => this.markerClick(e));
			}
			// marker.emit('click', {
			// 	target: marker
			// });
		})
	})
}


InitMap.prototype.addMarkerCustom = function(position, icon) {
	const marker = new this.mapInstance.Marker({
		position,
		map: this.map,
		// scale: 0.2,
		anchor: 'center',
		'offset': new this.mapInstance.Pixel(0, -12),
		// 'content': `<img src="${url}" style="width:36px;height:36px"/>`,
		icon,
	});

	return marker;
}


/**
 * @description 添加线段
 * @param {Object} path
 */
InitMap.prototype.addPolyLine = function(path, showEnd) {
	if (!path || !path.length) return;
	const start = path[0];
	const end = path[path.length - 1];

	if (this.polyLine) {
		this.polyLine.setPath(path);
	} else {
		this.polyLine = new this.mapInstance.Polyline({
			path: path, // 设置线覆盖物路径
			strokeColor: '#3366FF', // 线颜色
			strokeOpacity: 1, // 线透明度
			strokeWeight: 4, // 线宽
			lineJoin: 'round',
			lineCap: 'round',
			strokeStyle: 'solid', // 线样式
			// strokeDasharray: [10, 5], // 补充线样式
			showDir: false,
			geodesic: false, // 绘制大地线
		});
		this.polyLine.setMap(this.map);

		if (showEnd) {
			this.startMarker = this.addMarkerCustom(start, this.startIcon);
			this.endMarker = this.addMarkerCustom(end, this.endIcon);
		} else {
			console.log('eeee')
			this.startMarker = this.addMarkerCustom(start, this.startIcon);
		}
	}

	return this.polyLine;
}

InitMap.prototype.clearPolyLine = function() {

	if (this.startMarker) {
		this.map.remove(this.startMarker);
	}

	if (this.endMarker) {
		this.map.remove(this.endMarker);
	}

	if (this.polyLine) {
		this.map.remove(this.polyLine);
	}

	this.polyLine = null;
	this.endMarker = null;
	this.startMarker = null;
}


/**
 * @description 添加maker点详情
 * @param {Object} e maker点实例
 */
InitMap.prototype.markerClick = function(e) {
	const {
		_position
	} = e.target;
	this.map.setZoom(12);
	setTimeout(() => {
		this.map.panTo(_position);
	}, 0)
	// this.map.setCenter(_position)
	// this.map.setFitView([e.target]);
	this.infoWindow.setContent(e.target.content);
	this.infoWindow.open(this.map, e.target.getPosition());
}


/**
 * @description 清除maker点窗体
 */
InitMap.prototype.clearMarkerDetail = function() {
	this.infoWindow.close();
}

/**
 * @description 平移到指定位置
 * @param {Array} arr 经纬度坐标
 */
InitMap.prototype.panToLocation = function(arr) {
	this.map.panTo(arr || this.config.center)
}


/**
 * @description  设置地图事件
 */

InitMap.prototype.addEvent = function(type, callback) {
	if (this.eventHandle.has(type)) {
		this.map.off(type, this.eventHandle.get(type));
		this.eventHandle.delete(type)
	} else {
		this.eventHandle.set(type, callback)
		this.map.on(type, callback)
	}
}

/**
 * @description 初始化图层
 */
InitMap.prototype.initLayer = function() {
	const {
		options,
		mapInstance,
		map,
		layerSet
	} = this;
	const {
		defaultLayer,
		baseLayers
	} = options;

	const list = [];

	if (baseLayers) {
		baseLayers.forEach((item) => {
			const {
				config,
				name,
				visible
			} = item;

			config.forEach((key) => {
				let layer;

				if (key === 'default') {
					layer = new mapInstance.TileLayer();
					// layer.setMap(map);

				} else {
					layer = new mapInstance.TileLayer[key]();
					// layer.hide();
					// layer.setMap(map);
				}

				if (!this.layerSet.hasOwnProperty(name)) {
					this.layerSet[name] = {};
					this.layerSet[name].layers = [];
				}

				if (visible) {
					layer.show();
				} else {
					layer.hide();
				}

				this.layerSet[name].layers.push({
					key,
					layer
				});

				list.push(layer);
			});

		})
		// this.switchLayer(defaultLayer);
	}
	return list;
}


/**
 * @description 初始化图层
 */
InitMap.prototype.switchLayer = function(name) {
	if (!name) return;

	console.log(name)

	if (this.layerSet.hasOwnProperty(name)) {
		Object.entries(this.layerSet).forEach((item) => {
			const [key, value] = item;
			const {
				layers
			} = value;

			layers.forEach((i) => {
				if (key === name) {
					i.layer.show();
				} else {
					i.layer.hide();
				}
			})
		})
	} else {
		Object.entries(this.layerSet).forEach((item) => {
			const [key, value] = item;
			const {
				layers
			} = value;
			layers.forEach((i) => {
				i.layer.hide();
			})
		})
	}
}

/**
 * @description 加载离线地图
 */
InitMap.prototype.addOfflineLayer = function(filePath = '/offlineMap/gaode.sqlite') {
	// _jshost + 
	console.log(filePath)
	const isOpenDB = plus.sqlite.isOpenDatabase({
		name: 'gaode',
		path: filePath,
	});
	if (!isOpenDB) {
		plus.sqlite.openDatabase({
			name: 'gaode',
			path: filePath,
			success: (e) => {
				this.offlineLayer = new AMap.TileLayer.Flexible({
					createTile: function(x, y, z, success, fail) {
						plus.sqlite.selectSql({
							name: 'gaode',
							sql: `SELECT * FROM tiles where zoom_level = ${z} and tile_column = ${x} and tile_row = ${y} limit 1`,
							success: function(data) {
								if (data && data[0] && data[0].tile_data) {
									var img = document.createElement("img");
									img.onload = function() {
										// console.log('1111111' + ',' + x +
										// 	',' + y + ',' + z)
										success(img);
									};
									img.crossOrigin =
										"anonymous"; // 必须添加，同时图片要有跨域头
									img.onerror = function(err) {
										// console.log('2222222' + ',' + x + ',' + y + ',' + z)
										fail();
									};
									const imgData = []
									const originData = JSON.parse(data[0]
										.tile_data)
									for (var i = 0; i < originData
										.length; i++) {
										if (originData[i] < 0) {
											imgData.push(256 + originData[i])
										} else {
											imgData.push(originData[i])
										}
									}
									img.src =
										`data:image/png;base64,${arrayBuffer2Base64(imgData)}`;
								}
							},
							fail: function(e) {
								console.log('selectSql failed: ' + JSON
									.stringify(e));
							}
						});
					},
					cacheSize: 256, // 内存中缓存的切片的数量上限
					zIndex: 14,
					zooms: [4, 18], // 设置可见级别，[最小级别，最大级别]
				});
				this.map.add(this.offlineLayer)
			},
			fail: function(e) {
				console.log('openDatabase failed: ' + JSON.stringify(e));
			}
		});
	} else {
		this.offlineLayer = new AMap.TileLayer.Flexible({
			createTile: function(x, y, z, success, fail) {
				plus.sqlite.selectSql({
					name: 'gaode',
					sql: `SELECT * FROM tiles where zoom_level = ${z} and tile_column = ${x} and tile_row = ${y} limit 1`,
					success: function(data) {
						if (data && data[0] && data[0].tile_data) {
							var img = document.createElement("img");
							img.onload = function() {
								// console.log('1111111' + ',' + x + ',' + y + ',' + z)
								success(img);
							};
							img.crossOrigin = "anonymous"; // 必须添加，同时图片要有跨域头
							img.onerror = function(err) {
								// console.log('2222222' + ',' + x + ',' + y + ',' + z)
								fail();
							};
							const imgData = []
							const originData = JSON.parse(data[0].tile_data)
							for (var i = 0; i < originData.length; i++) {
								if (originData[i] < 0) {
									imgData.push(256 + originData[i])
								} else {
									imgData.push(originData[i])
								}
							}
							img.src =
								`data:image/png;base64,${arrayBuffer2Base64(imgData)}`;
						}
					},
					fail: function(e) {
						console.log('selectSql failed: ' + JSON.stringify(e));
					}
				});

			},
			cacheSize: 256, // 内存中缓存的切片的数量上限
			zIndex: 14,
			zooms: [4, 18], // 设置可见级别，[最小级别，最大级别]
		});
		this.map.add(this.offlineLayer)
	}
}
/**
 * @description 移除离线地图
 */
InitMap.prototype.removeOfflineLayer = function() {
	if (this.offlineLayer) {
		this.map.remove(this.offlineLayer)
		this.offlineLayer = null
		plus.sqlite.closeDatabase({
			name: 'gaode'
		});
	}
}
