/**
 * 打开定位导航
 */
function toNavigation(lon, lat) {
	if (lon == '' || lat == '' || lon == null || lat == null) {
		mui.toast('导航失败!');
	} else {
		mui.toast("进入地图导航");
		//设置终点位置信息，进行偏移矫正
		var newLon = lon * 1 - 0.01235;
		var newLat = lat * 1 - 0.0065;
		var point = new plus.maps.Point(newLon, newLat);
		//var point = new plus.maps.Point(114.522076, 38.048958);
		plus.maps.Map.reverseGeocode(point, {
			coordType: "WGS84",
			city: '石家庄市'
		}, function(event) {
			var address = event.address; // 转换后的地理位置
			var points = event.coord;
			var src = new plus.maps.Point(115.3626242, 28.8614724);
			console.log('point=' + JSON.stringify(point));
			console.log('address=' + address);
			console.log('src=' + JSON.stringify(src));
			plus.maps.openSysMap(point, address, src);
		}, function(e) {
			mui.toast(JSON.stringify(e));
		});
	}
}

/**
 * 绘制静安县矢量图形
 */
function setGeo(url) {
	ajax(url, function(err, geoJSON) {
		if (!err) {
			// console.log('GeoJSON 数据加载完成===' + JSON.stringify(geoJSON))
			geojson = new AMap.GeoJSON({
				geoJSON: geoJSON,
				// 还可以自定义getMarker和getPolyline
				getPolygon: function(geojson, lnglats) {
					// 计算面积
					var area = AMap.GeometryUtil.ringArea(lnglats[0])

					return new AMap.Polygon({
						path: lnglats,
						fillOpacity: 1 - Math.sqrt(area / 8000000000), // 面积越大透明度越高
						strokeColor: '#0055ff',
						fillColor: '#55b4ff',
						strokeWeight: 2,
						strokeOpacity: 0.8,
						fillOpacity: 0.3,
						strokeStyle: 'dashed',
					});
				}
			});
			geojson.setMap(map);
			geojson.on('click', function(e) {
				if (marker) {
					map.remove(marker);
				}
				console.log(e.lnglat.getLng() + ',' + e.lnglat.getLat())
				marker = new AMap.Marker({
					map: map,
					position: [e.lnglat.getLng(), e.lnglat.getLat()],
				});
				marker.on('dragging', showInfoM);
				marker.on('click', function(e) {
					var address;
					var lnglatXY = [e.lnglat.getLng(), e.lnglat.getLat()];
					geocoder.getAddress(lnglatXY, function(status, result) {
						if (status === 'complete' && result.info === 'OK') {
							address = result.regeocode.formattedAddress; //返回地址描述
							mui.fire(targetPage, 'eventLocation', {
								address: address,
								lng: e.lnglat.getLng(),
								lat: e.lnglat.getLat()
							});
							ws.close();
						}
					});
				});
			});
			console.log('GeoJSON 数据加载完成')
		} else {
			console.log('GeoJSON 服务请求失败')
		}
	})
}

/**
 * marker拖拽事件
 * @param {Object} e
 */
function showInfoM(e) {
	var text = '您拖拽了marker！'
	lnglatXY = [e.lnglat.getLng(), e.lnglat.getLat()];
	mui.toast(text + lnglatXY);
	var geocoder = new AMap.Geocoder({
		radius: 1000,
		extensions: "all"
	});
	geocoder.getAddress(lnglatXY, function(status, result) {
		if (status === 'complete' && result.info === 'OK') {
			var address = result.regeocode.formattedAddress; //返回地址描述
			mui.toast(address);
		}
	});
}

/**
 * 从当前页面pop到目标页面
 * @param {String} targetId 目标页面ID
 */
function popToTarget(targetId) {
	//获取目标页面
	var target = plus.webview.getWebviewById(targetId);
	if (!target) {
		console.log("目标页面不存在！");
		return;
	}
	//获取当前页面
	var current = plus.webview.currentWebview();
	if (current === target) {
		console.log("目标页面是当前页面！");
		return;
	}
	//将要关闭的页面
	var pages = new Array(current);
	//父级页面
	var opener = current.opener();
	while (opener) {
		if (opener === target) { //找到了目标页面
			//关闭目标页面的所有子级页面pages
			pages.map(function(page) {
				page.close();
			});
			return;
		}
		pages.push(opener);
		opener = opener.opener();
	}
	//没有找到目标页面
	console.log("目标页面不是当前页面的祖先页面！");
}
